__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from .acs_read_education import AcsReadEducation
from .acs_read_income import AcsReadIncome
from .acs_read_race import AcsReadRace
from .acs_read_age65 import AcsReadAge65
from .acs_read_total_population import AcsReadTotalPopulation
from .acs_read_type_health_insurance_age import AcsReadTypeHealthInsuranceAge

ALL_IMPLEMENTATIONS = [
    AcsReadEducation,
    AcsReadIncome,
    AcsReadRace,
    AcsReadAge65,
    AcsReadTotalPopulation,
    AcsReadTypeHealthInsuranceAge
]



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""


class Parameter(object):

    def __init__(self):
        pass

    @staticmethod
    def set_par(par, name, func, default=None, transform=None):
        par = Parameter.get_par(par, name, default, transform)
        if par:
            func(par)

    # @staticmethod
    # def set_par(par_plot, names, func, default=None, transform=None):
    #     if not isinstance(names, list):
    #         names = [names]
    #
    #     pars = []
    #     for n in names:
    #         par = PlotBase.get_par(par_plot, n, default, transform)
    #         pars.append(par)
    #     if pars:
    #         func(tuple(pars))

    @staticmethod
    def get_par(par, name, default=None, transform=None):
        param = None
        if par.keys().__contains__(name):
            param = par[name]
        elif not par.keys().__contains__(name) and default is not None:
            param = default
        if (param is not None) and (transform is not None):
            param = transform(param)
        return param



__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadRace(AcsRead):

    def __init__(self, datapath):
        super().__init__(datapath=datapath, prefix='RAC', datafile='B03002', column_format='HD01_VD%02d',
                         is_percent=False)

    def get_mappings(self):
        """
        HD01_VD03
        Estimate; Not Hispanic or Latino: - White alone

        HD01_VD04
        Estimate; Not Hispanic or Latino: - Black or African American alone
        """
        mappings = [{
            'TOTAL': 1,
            'WHITE': 3,
            'BLACK': 4,
            'INDIAN': 5,
            'ASIAN': 6,
            'PACIFIC': 7,
            'OTHER': [8, 9],
            'HISPANIC': [13, 14, 15, 16, 17, 18, 19]
        }]
        return mappings



__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadTotalPopulation(AcsRead):

    def __init__(self, datapath):
        super().__init__(datapath=datapath, prefix='TOT', datafile='B01003', column_format='HD01_VD%02d')

    def get_mappings(self):
        mappings = [
            {
                'TOTAL_POPULATION': [1]
            }
        ]
        return mappings



__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadEducation(AcsRead):

    # def __init__(self, datapath, year, estimates):
    def __init__(self, datapath):
        # super().__init__(datapath=datapath, prefix='EDU', datafile='S1501', year=year, estimates=estimates,
        #                  column_format='HC01_EST_VC%02d', is_percent=False)
        super().__init__(datapath=datapath, prefix='EDU', datafile='S1501', column_format='HC01_EST_VC%02d',
                         is_percent=False)

    def get_mappings(self):
        """
        HC01_EST_VC02
        Total; Estimate; Population 18 to 24 years

        HC01_EST_VC03
        Total; Estimate; Population 18 to 24 years - Less than high school graduate

        HC01_EST_VC04
        Total; Estimate; Population 18 to 24 years - High school graduate (includes equivalency)

        HC01_EST_VC05
        Total; Estimate; Population 18 to 24 years - Some college or associate's degree

        HC01_EST_VC06
        Total; Estimate; Population 18 to 24 years - Bachelor's degree or higher

        HC01_EST_VC08
        Total; Estimate; Population 25 years and over

        HC01_EST_VC09
        Total; Estimate; Population 25 years and over - Less than 9th grade

        HC01_EST_VC10
        Total; Estimate; Population 25 years and over - 9th to 12th grade, no diploma

        HC01_EST_VC11
        Total; Estimate; Population 25 years and over - High school graduate (includes equivalency)

        HC01_EST_VC12
        Total; Estimate; Population 25 years and over - Some college, no degree

        HC01_EST_VC13
        Total; Estimate; Population 25 years and over - Associate's degree

        HC01_EST_VC14
        Total; Estimate; Population 25 years and over - Bachelor's degree

        HC01_EST_VC15
        Total; Estimate; Population 25 years and over - Graduate or professional degree
        """

        mappings = [
            {
                'TOTAL': [2],
                'ELEMENTAR': [3],
                'HIGH': [4],
                'COLLEGE': [5],
                'BACHELOR': [6]
            },
            {
                'TOTAL': [8],
                'ELEMENTAR': [9],
                'HIGH': [10, 11],
                'COLLEGE': [12],
                'BACHELOR': [13, 14, 15]
            }
        ]
        return mappings



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""
import itertools as it
from old.script import AcsPreprocess
from acs_wrapper.src.util_multiprocessing import MyPool


class AcsProcess(object):

    def __init__(self, filepath_input, filepath_output, ):
        self.filepath_input = filepath_input
        self.filepath_output = filepath_output

    def write_hdf(self, args):
        reader, level, year, estimate = args
        reader = reader(self.filepath_input)
        df = reader.get_data(level, year, estimate)
        filename_output = f'{self.filepath_output}/df_{reader.prefix}_{level}_{year}_{estimate}'
        df.to_csv(f'{filename_output}.csv', index=True)
        df.to_hdf(f'{filename_output}.hdf', 'df')

    def process(self):
        list_reader, list_level, list_year, list_estimate = AcsPreprocess.get_param()
        list_args = it.product(list_reader, list_level, list_year, list_estimate)
        with MyPool(processes=4) as pool:
            pool.map(self.write_hdf, list_args)

# def main():
#     print('INITIALIZING EXPORTING ACS DATA')
#     list_reader, list_level, list_year, list_estimate = AcsPreprocess.get_param()
#
#     mod_acs_process = AcsProcess()
#     # data = dict()
#     # for arg in it.product(list_reader, list_level, list_year, list_estimate):
#         # name = reader.__name__.split('AcsRead')[1].lower()
#         # data[f'{name}_{level}_{year}_{estimate}'] = reader(datapath=datapath % level, year=year,
#         #                                                               estimates=estimate)
#     # list_args = tuple(zip(data.keys(), data.values()))
#     list_args = it.product(list_reader, list_level, list_year, list_estimate)
#
#
#     print('FINISHED EXPORTING ACS DATA')


# if __name__ == "__main__":
#     main()



__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadTypeHealthInsuranceAge(AcsRead):

    def __init__(self, datapath):
        super().__init__(datapath=datapath, prefix='AGE', datafile='S0101', column_format='HD02_VD%02d',
                         is_percent=True)

    def get_mappings(self):
        """
        HC01_EST_VC01
        Total; Estimate; Total population

        HC01_EST_VC03
        Total; Estimate; AGE - Under 5 years

        HC01_EST_VC04
        Total; Estimate; AGE - 5 to 9 years

        HC01_EST_VC05
        Total; Estimate; AGE - 10 to 14 years

        HC01_EST_VC06
        Total; Estimate; AGE - 15 to 19 years

        HC01_EST_VC07
        Total; Estimate; AGE - 20 to 24 years

        HC01_EST_VC08
        Total; Estimate; AGE - 25 to 29 years

        HC01_EST_VC09
        Total; Estimate; AGE - 30 to 34 years

        HC01_EST_VC10
        Total; Estimate; AGE - 35 to 39 years

        HC01_EST_VC11
        Total; Estimate; AGE - 40 to 44 years

        HC01_EST_VC12
        Total; Estimate; AGE - 45 to 49 years

        HC01_EST_VC13
        Total; Estimate; AGE - 50 to 54 years

        HC01_EST_VC14
        Total; Estimate; AGE - 55 to 59 years

        HC01_EST_VC15
        Total; Estimate; AGE - 60 to 64 years

        HC01_EST_VC16
        Total; Estimate; AGE - 65 to 69 years

        HC01_EST_VC17
        Total; Estimate; AGE - 70 to 74 years

        HC01_EST_VC18
        Total; Estimate; AGE - 75 to 79 years

        HC01_EST_VC19
        Total; Estimate; AGE - 80 to 84 years

        HC01_EST_VC20
        Total; Estimate; AGE - 85 years and over

        """
        mappings = [
            {
                'TOTAL': [1],
                'UNDER_05': [3],
                'FROM_05_TO_09': [4],
                'FROM_10_TO_14': [5],
                'FROM_15_TO_19': [6],
                'FROM_20_TO_24': [7],
                'FROM_25_TO_29': [8],
                'FROM_30_TO_34': [9],
                'FROM_35_TO_39': [10],
                'FROM_40_TO_44': [11],
                'FROM_45_TO_49': [12],
                'FROM_50_TO_54': [13],
                'FROM_55_TO_59': [14],
                'FROM_60_TO_64': [15],
                'FROM_65_TO_69': [16],
                'FROM_70_TO_74': [17],
                'FROM_75_TO_79': [18],
                'FROM_80_TO_84': [19],
                'ABOVE_85': [20],

            }
        ]
        return mappings



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

import pandas as pd
import numpy as np
import multiprocessing


class ReadFilePandas(object):
    __ENCODING__ = "ISO-8859-1"

    def __init__(self, filename, encoding=__ENCODING__):
        self.filename = filename
        self.encoding = encoding

    def get_data(self):
        column_mapping = self.get_column_mapping()
        # df = pd.read_table(self.filename, delimiter='\t', header=0, encoding=self.encoding, nrows=1000)
        df = pd.read_table(self.filename, delimiter='\t', header=0, encoding=self.encoding)
        df.columns = [e.strip() for e in list(df.columns)]
        if column_mapping is not None:
            df = df[list(column_mapping.values())]
            df = df.rename(columns=dict(zip(column_mapping.values(), column_mapping.keys())))
        df = apply_by_multiprocessing(df, self.pre_process, axis=1, workers=4)
        return df

    def _process(self, row):
        self.pre_process(row)
        self.post_process(row)
        return row

    def pre_process(self, row):
        return row

    def post_process(self, row):
        return row

    def get_column_mapping(self):
        pass


def apply_by_multiprocessing(df, func, **kwargs):
    workers = kwargs.pop('workers')

    with multiprocessing.Pool(processes=workers) as pool:
        result = pool.map(_apply_df, [(d, func, kwargs)
                                      for d in np.array_split(df, workers)])
        return pd.concat(list(result))
    # pool = multiprocessing.Pool(processes=workers)
    #     return
    # pool.close()


def _apply_df(args):
    df, func, kwargs = args
    return df.apply(func, **kwargs)



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

from acs_wrapper.src.read.acs_read_base import AcsRead




#
# datapath = 'data/data_input/%s'
# l = 'county'
# y = 16
# e = 5
# r = AcsReadRace(datapath=datapath % l, year=y, estimates=e)
# r_edu = AcsReadEducation(datapath=datapath % l, year=y, estimates=e)
# r_inc = AcsReadIncome(datapath=datapath % l, year=y, estimates=e)
#
# df_rac = r.get_data()
# df_edu = r_edu.get_data()
# df_inc = r_inc.get_data()
#
#
# df_rac.loc['01001']
# df_edu.loc['01001']
# df_inc.loc['01001']
#
#
# import numpy as np
# np.corrcoef(df_edu.loc[:, 'EDU_TOTAL'].values, df_rac.loc[:, 'RAC_TOTAL'].values)
#
#
